@@ -1,11 +1,3 @@
-#' @importFrom fastmap key_missing
-#' @export
-fastmap::key_missing
-
-#' @importFrom fastmap is.key_missing
-#' @export
-fastmap::is.key_missing
-
 
 validate_key <- function(key) {
   if (!is.character(key) || length(key) != 1 || nchar(key) == 0) {
@@ -15,4 +7,3 @@ validate_key <- function(key) {
     stop("Invalid key: ", key, ". Only lowercase letters and numbers are allowed.")
   }
 }
-
