@@ -22,6 +22,7 @@ registerClient <- function(client) {
 }
 
 
+.globals$resourcePaths <- list()
 .globals$resources <- list()
 
 .globals$showcaseDefault <- 0
@@ -30,29 +31,46 @@ registerClient <- function(client) {
 
 #' Resource Publishing
 #'
-#' Adds a directory of static resources to Shiny's web server, with the given
-#' path prefix. Primarily intended for package authors to make supporting
-#' JavaScript/CSS files available to their components.
+#' Add, remove, or list directory of static resources to Shiny's web server,
+#' with the given path prefix. Primarily intended for package authors to make
+#' supporting JavaScript/CSS files available to their components.
+#'
+#' Shiny provides two ways of serving static files (i.e., resources):
+#'
+#' 1. Static files under the `www/` directory are automatically made available
+#' under a request path that begins with `/`.
+#' 2. `addResourcePath()` makes static files in a `directoryPath` available
+#' under a request path that begins with `prefix`.
+#'
+#' The second approach is primarily intended for package authors to make
+#' supporting JavaScript/CSS files available to their components.
+#'
+#' Tools for managing static resources published by Shiny's web server:
+#'  * `addResourcePath()` adds a directory of static resources.
+#'  * `resourcePaths()` lists the currently active resource mappings.
+#'  * `removeResourcePath()` removes a directory of static resources.
 #'
 #' @param prefix The URL prefix (without slashes). Valid characters are a-z,
-#'   A-Z, 0-9, hyphen, period, and underscore.
-#'   For example, a value of 'foo' means that any request paths that begin with
-#'   '/foo' will be mapped to the given directory.
+#'   A-Z, 0-9, hyphen, period, and underscore. For example, a value of 'foo'
+#'   means that any request paths that begin with '/foo' will be mapped to the
+#'   given directory.
 #' @param directoryPath The directory that contains the static resources to be
 #'   served.
 #'
-#' @details You can call \code{addResourcePath} multiple times for a given
-#'   \code{prefix}; only the most recent value will be retained. If the
-#'   normalized \code{directoryPath} is different than the directory that's
-#'   currently mapped to the \code{prefix}, a warning will be issued.
-#'
-#' @seealso \code{\link{singleton}}
+#' @rdname resourcePaths
+#' @seealso [singleton()]
 #'
 #' @examples
 #' addResourcePath('datasets', system.file('data', package='datasets'))
+#' resourcePaths()
+#' removeResourcePath('datasets')
+#' resourcePaths()
+#'
+#' # make sure all resources are removed
+#' lapply(names(resourcePaths()), removeResourcePath)
 #' @export
 addResourcePath <- function(prefix, directoryPath) {
-  prefix <- prefix[1]
+  if (length(prefix) != 1) stop("prefix must be of length 1")
   if (!grepl('^[a-z0-9\\-_][a-z0-9\\-_.]*$', prefix, ignore.case = TRUE, perl = TRUE)) {
     stop("addResourcePath called with invalid prefix; please see documentation")
   }
@@ -66,30 +84,112 @@ addResourcePath <- function(prefix, directoryPath) {
         "`prefix` = '", prefix, "'; `directoryPath` = '" , directoryPath, "'")
     }
   )
+
+  # # Often times overwriting a resource path is "what you want",
+  # # but sometimes it can lead to difficult to diagnose issues
+  # # (e.g. an implict dependency might set a resource path that
+  # # conflicts with what you, the app author, are trying to register)
+  # # Note that previous versions of shiny used to warn about this case,
+  # # but it was eventually removed since it caused confusion (#567).
+  # # It seems a good compromise is to throw a more information message.
+  # if (getOption("shiny.resourcePathChanges", FALSE) &&
+  #     prefix %in% names(.globals$resourcePaths)) {
+  #   existingPath <- .globals$resourcePaths[[prefix]]$path
+  #   if (normalizedPath != existingPath) {
+  #     message(
+  #       "The resource path '", prefix, "' used to point to ",
+  #       existingPath, ", but it now points to ", normalizedPath, ". ",
+  #       "If your app doesn't work as expected, you may want to ",
+  #       "choose a different prefix name."
+  #     )
+  #   }
+  # }
+
+  # If a shiny app is currently running, dynamically register this path with
+  # the corresponding httpuv server object.
+  if (!is.null(getShinyOption("server")))
+  {
+    getShinyOption("server")$setStaticPath(.list = stats::setNames(normalizedPath, prefix))
+  }
+
+  # .globals$resourcePaths and .globals$resources persist across runs of applications.
+  .globals$resourcePaths[[prefix]] <- staticPath(normalizedPath)
+  # This is necessary because resourcePaths is only for serving assets out of C++;
+  # to support subapps, we also need assets to be served out of R, because those
+  # URLs are rewritten by R code (i.e. routeHandler) before they can be matched to
+  # a resource path.
   .globals$resources[[prefix]] <- list(
     directoryPath = normalizedPath,
     func = staticHandler(normalizedPath)
   )
 }
 
+#' @rdname resourcePaths
+#' @export
+resourcePaths <- function() {
+  urls <- names(.globals$resourcePaths)
+  paths <- vapply(.globals$resourcePaths, function(x) x$path, character(1))
+  stats::setNames(paths, urls)
+}
+
+hasResourcePath <- function(prefix) {
+  prefix %in% names(resourcePaths())
+}
+
+#' @rdname resourcePaths
+#' @export
+removeResourcePath <- function(prefix) {
+  if (length(prefix) > 1) stop("`prefix` must be of length 1.")
+  if (!hasResourcePath(prefix)) {
+    warning("Resource ", prefix, " not found.")
+    return(invisible(FALSE))
+  }
+  .globals$resourcePaths[[prefix]] <- NULL
+  .globals$resources[[prefix]] <- NULL
+  invisible(TRUE)
+}
+
+
+
+# This function handles any GET request with two or more path elements where the
+# first path element matches a prefix that was previously added using
+# addResourcePath().
+#
+# For example, if `addResourcePath("foo", "~/bar")` was called, then a GET
+# request for /foo/one/two.html would rewrite the PATH_INFO as /one/two.html and
+# send it to the resource path function for "foo". As of this writing, that
+# function will always be a staticHandler, which serves up a file if it exists
+# and NULL if it does not.
+#
+# Since Shiny 1.3.x, assets registered via addResourcePath should mostly be
+# served out of httpuv's native static file serving features. However, in the
+# specific case of subapps, the R code path must be used, because subapps insert
+# a giant random ID into the beginning of the URL that must be stripped off by
+# an R route handler (see addSubApp()).
 resourcePathHandler <- function(req) {
   if (!identical(req$REQUEST_METHOD, 'GET'))
     return(NULL)
 
+  # e.g. "/foo/one/two.html"
   path <- req$PATH_INFO
 
   match <- regexpr('^/([^/]+)/', path, perl=TRUE)
   if (match == -1)
     return(NULL)
   len <- attr(match, 'capture.length')
+  # e.g. "foo"
   prefix <- substr(path, 2, 2 + len - 1)
 
   resInfo <- .globals$resources[[prefix]]
   if (is.null(resInfo))
     return(NULL)
 
+  # e.g. "/one/two.html"
   suffix <- substr(path, 2 + len, nchar(path))
 
+  # Create a new request that's a clone of the current request, but adjust
+  # PATH_INFO and SCRIPT_NAME to reflect that we have already matched the first
+  # path element (e.g. "/foo"). See routeHandler() for more info.
   subreq <- as.environment(as.list(req, all.names=TRUE))
   subreq$PATH_INFO <- suffix
   subreq$SCRIPT_NAME <- paste(subreq$SCRIPT_NAME, substr(path, 1, 2 + len), sep='')
@@ -101,23 +201,23 @@ resourcePathHandler <- function(req) {
 #'
 #' Defines the server-side logic of the Shiny application. This generally
 #' involves creating functions that map user inputs to various kinds of output.
-#' In older versions of Shiny, it was necessary to call \code{shinyServer()} in
-#' the \code{server.R} file, but this is no longer required as of Shiny 0.10.
-#' Now the \code{server.R} file may simply return the appropriate server
+#' In older versions of Shiny, it was necessary to call `shinyServer()` in
+#' the `server.R` file, but this is no longer required as of Shiny 0.10.
+#' Now the `server.R` file may simply return the appropriate server
 #' function (as the last expression in the code), without calling
-#' \code{shinyServer()}.
+#' `shinyServer()`.
 #'
-#' Call \code{shinyServer} from your application's \code{server.R}
+#' Call `shinyServer` from your application's `server.R`
 #' file, passing in a "server function" that provides the server-side logic of
 #' your application.
 #'
 #' The server function will be called when each client (web browser) first loads
-#' the Shiny application's page. It must take an \code{input} and an
-#' \code{output} parameter. Any return value will be ignored. It also takes an
-#' optional \code{session} parameter, which is used when greater control is
+#' the Shiny application's page. It must take an `input` and an
+#' `output` parameter. Any return value will be ignored. It also takes an
+#' optional `session` parameter, which is used when greater control is
 #' needed.
 #'
-#' See the \href{http://rstudio.github.com/shiny/tutorial/}{tutorial} for more
+#' See the [tutorial](http://rstudio.github.com/shiny/tutorial/) for more
 #' on how to write a server function.
 #'
 #' @param func The server function for this application. See the details section
@@ -144,6 +244,7 @@ resourcePathHandler <- function(req) {
 #' }
 #' }
 #' @export
+#' @keywords internal
 shinyServer <- function(func) {
   .globals$server <- list(func)
   invisible(func)
@@ -178,6 +279,8 @@ decodeMessage <- function(data) {
   return(mainMessage)
 }
 
+autoReloadCallbacks <- Callbacks$new()
+
 createAppHandlers <- function(httpHandlers, serverFuncSource) {
   appvars <- new.env()
   appvars$server <- NULL
@@ -187,7 +290,7 @@ createAppHandlers <- function(httpHandlers, serverFuncSource) {
   # This value, if non-NULL, must be present on all HTTP and WebSocket
   # requests as the Shiny-Shared-Secret header or else access will be
   # denied (403 response for HTTP, and instant close for websocket).
-  sharedSecret <- getOption('shiny.sharedSecret')
+  checkSharedSecret <- loadSharedSecret()
 
   appHandlers <- list(
     http = joinHandlers(c(
@@ -195,14 +298,30 @@ createAppHandlers <- function(httpHandlers, serverFuncSource) {
       httpHandlers,
       sys.www.root,
       resourcePathHandler,
-      reactLogHandler)),
+      reactLogHandler
+    )),
     ws = function(ws) {
-      if (!is.null(sharedSecret)
-          && !identical(sharedSecret, ws$request$HTTP_SHINY_SHARED_SECRET)) {
+      if (!checkSharedSecret(ws$request$HTTP_SHINY_SHARED_SECRET)) {
         ws$close()
         return(TRUE)
       }
 
+      if (identical(ws$request$PATH_INFO, "/autoreload/")) {
+        if (!getOption("shiny.autoreload", FALSE)) {
+          ws$close()
+          return(TRUE)
+        }
+
+        callbackHandle <- autoReloadCallbacks$register(function() {
+          ws$send("autoreload")
+          ws$close()
+        })
+        ws$onClose(function() {
+          callbackHandle()
+        })
+        return(TRUE)
+      }
+
       if (!is.null(getOption("shiny.observer.error", NULL))) {
         warning(
           call. = FALSE,
@@ -417,6 +536,70 @@ startApp <- function(appObj, port, host, quiet) {
   handlerManager$addHandler(appHandlers$http, "/", tail = TRUE)
   handlerManager$addWSHandler(appHandlers$ws, "/", tail = TRUE)
 
+  httpuvApp <- handlerManager$createHttpuvApp()
+  httpuvApp$staticPaths <- c(
+    appObj$staticPaths,
+    list(
+      # Always handle /session URLs dynamically, even if / is a static path.
+      "session" = excludeStaticPath(),
+      "shared" = system.file(package = "shiny", "www", "shared")
+    ),
+    .globals$resourcePaths
+  )
+
+  # throw an informative warning if a subdirectory of the
+  # app's www dir conflicts with another resource prefix
+  wwwDir <- httpuvApp$staticPaths[["/"]]$path
+  if (length(wwwDir)) {
+    # although httpuv allows for resource prefixes like 'foo/bar',
+    # we won't worry about conflicts in sub-sub directories since
+    # addResourcePath() currently doesn't allow it
+    wwwSubDirs <- list.dirs(wwwDir, recursive = FALSE, full.names = FALSE)
+    resourceConflicts <- intersect(wwwSubDirs, names(httpuvApp$staticPaths))
+    if (length(resourceConflicts)) {
+      warning(
+        "Found subdirectories of your app's www/ directory that ",
+        "conflict with other resource URL prefixes. ",
+        "Consider renaming these directories: '",
+        paste0("www/", resourceConflicts, collapse = "', '"), "'",
+        call. = FALSE
+      )
+    }
+  }
+
+  # check for conflicts in each pairwise combinations of resource mappings
+  checkResourceConflict <- function(paths) {
+    if (length(paths) < 2) return(NULL)
+    # ensure paths is a named character vector: c(resource_path = local_path)
+    paths <- vapply(paths, function(x) if (inherits(x, "staticPath")) x$path else x, character(1))
+    # get all possible pairwise combinations of paths
+    pair_indices <- utils::combn(length(paths), 2, simplify = FALSE)
+    lapply(pair_indices, function(x) {
+      p1 <- paths[x[1]]
+      p2 <- paths[x[2]]
+      if (identical(names(p1), names(p2)) && (p1 != p2)) {
+        warning(
+          "Found multiple local file paths pointing the same resource prefix: ", names(p1), ". ",
+          "If you run into resource-related issues (e.g. 404 requests), consider ",
+          "using `addResourcePath()` and/or `removeResourcePath()` to manage resource mappings.",
+          call. = FALSE
+        )
+      }
+    })
+  }
+  checkResourceConflict(httpuvApp$staticPaths)
+
+  httpuvApp$staticPathOptions <- httpuv::staticPathOptions(
+    html_charset = "utf-8",
+    headers = list("X-UA-Compatible" = "IE=edge,chrome=1"),
+    validation =
+      if (!is.null(getOption("shiny.sharedSecret"))) {
+        sprintf('"Shiny-Shared-Secret" == "%s"', getOption("shiny.sharedSecret"))
+      } else {
+        character(0)
+      }
+  )
+
   if (is.numeric(port) || is.integer(port)) {
     if (!quiet) {
       hostString <- host
@@ -424,7 +607,7 @@ startApp <- function(appObj, port, host, quiet) {
         hostString <- paste0("[", hostString, "]")
       message('\n', 'Listening on http://', hostString, ':', port)
     }
-    return(startServer(host, port, handlerManager$createHttpuvApp()))
+    return(startServer(host, port, httpuvApp))
   } else if (is.character(port)) {
     if (!quiet) {
       message('\n', 'Listening on domain socket ', port)
@@ -436,7 +619,7 @@ startApp <- function(appObj, port, host, quiet) {
            "configuration (and not domain sockets), then `port` must ",
            "be numeric, not a string.")
     }
-    return(startPipeServer(port, mask, handlerManager$createHttpuvApp()))
+    return(startPipeServer(port, mask, httpuvApp))
   }
 }
 
@@ -468,8 +651,8 @@ serviceApp <- function() {
 #'
 #' This function tests whether a Shiny application is currently running.
 #'
-#' @return \code{TRUE} if a Shiny application is currently running. Otherwise,
-#'   \code{FALSE}.
+#' @return `TRUE` if a Shiny application is currently running. Otherwise,
+#'   `FALSE`.
 #' @export
 isRunning <- function() {
   .globals$running
@@ -481,44 +664,44 @@ isRunning <- function() {
 #' to stop the application (usually by pressing Ctrl+C or Esc).
 #'
 #' The host parameter was introduced in Shiny 0.9.0. Its default value of
-#' \code{"127.0.0.1"} means that, contrary to previous versions of Shiny, only
+#' `"127.0.0.1"` means that, contrary to previous versions of Shiny, only
 #' the current machine can access locally hosted Shiny apps. To allow other
-#' clients to connect, use the value \code{"0.0.0.0"} instead (which was the
+#' clients to connect, use the value `"0.0.0.0"` instead (which was the
 #' value that was hard-coded into Shiny in 0.8.0 and earlier).
 #'
 #' @param appDir The application to run. Should be one of the following:
 #'   \itemize{
-#'   \item A directory containing \code{server.R}, plus, either \code{ui.R} or
-#'    a \code{www} directory that contains the file \code{index.html}.
-#'   \item A directory containing \code{app.R}.
-#'   \item An \code{.R} file containing a Shiny application, ending with an
+#'   \item A directory containing `server.R`, plus, either `ui.R` or
+#'    a `www` directory that contains the file `index.html`.
+#'   \item A directory containing `app.R`.
+#'   \item An `.R` file containing a Shiny application, ending with an
 #'    expression that produces a Shiny app object.
-#'   \item A list with \code{ui} and \code{server} components.
-#'   \item A Shiny app object created by \code{\link{shinyApp}}.
+#'   \item A list with `ui` and `server` components.
+#'   \item A Shiny app object created by [shinyApp()].
 #'   }
 #' @param port The TCP port that the application should listen on. If the
-#'   \code{port} is not specified, and the \code{shiny.port} option is set (with
-#'   \code{options(shiny.port = XX)}), then that port will be used. Otherwise,
+#'   `port` is not specified, and the `shiny.port` option is set (with
+#'   `options(shiny.port = XX)`), then that port will be used. Otherwise,
 #'   use a random port.
 #' @param launch.browser If true, the system's default web browser will be
 #'   launched automatically after the app is started. Defaults to true in
 #'   interactive sessions only. This value of this parameter can also be a
 #'   function to call with the application's URL.
 #' @param host The IPv4 address that the application should listen on. Defaults
-#'   to the \code{shiny.host} option, if set, or \code{"127.0.0.1"} if not. See
+#'   to the `shiny.host` option, if set, or `"127.0.0.1"` if not. See
 #'   Details.
 #' @param workerId Can generally be ignored. Exists to help some editions of
 #'   Shiny Server Pro route requests to the correct process.
 #' @param quiet Should Shiny status messages be shown? Defaults to FALSE.
 #' @param display.mode The mode in which to display the application. If set to
-#'   the value \code{"showcase"}, shows application code and metadata from a
-#'   \code{DESCRIPTION} file in the application directory alongside the
-#'   application. If set to \code{"normal"}, displays the application normally.
-#'   Defaults to \code{"auto"}, which displays the application in the mode given
-#'   in its \code{DESCRIPTION} file, if any.
+#'   the value `"showcase"`, shows application code and metadata from a
+#'   `DESCRIPTION` file in the application directory alongside the
+#'   application. If set to `"normal"`, displays the application normally.
+#'   Defaults to `"auto"`, which displays the application in the mode given
+#'   in its `DESCRIPTION` file, if any.
 #' @param test.mode Should the application be launched in test mode? This is
 #'   only used for recording or running automated tests. Defaults to the
-#'   \code{shiny.testmode} option, or FALSE if the option is not set.
+#'   `shiny.testmode` option, or FALSE if the option is not set.
 #'
 #' @examples
 #' \dontrun{
@@ -777,6 +960,10 @@ runApp <- function(appDir=getwd(),
 
   server <- startApp(appParts, port, host, quiet)
 
+  # Make the httpuv server object accessible. Needed for calling
+  # addResourcePath while app is running.
+  shinyOptions(server = server)
+
   on.exit({
     stopServer(server)
   }, add = TRUE)
@@ -818,7 +1005,6 @@ runApp <- function(appDir=getwd(),
     captureStackTraces({
       while (!.globals$stopped) {
         ..stacktracefloor..(serviceApp())
-        Sys.sleep(0.001)
       }
     })
   )
@@ -835,10 +1021,10 @@ runApp <- function(appDir=getwd(),
 #' Stop the currently running Shiny app
 #'
 #' Stops the currently running Shiny app, returning control to the caller of
-#' \code{\link{runApp}}.
+#' [runApp()].
 #'
 #' @param returnValue The value that should be returned from
-#'   \code{\link{runApp}}.
+#'   [runApp()].
 #' @export
 stopApp <- function(returnValue = invisible()) {
   # reterror will indicate whether retval is an error (i.e. it should be passed
@@ -867,7 +1053,7 @@ stopApp <- function(returnValue = invisible()) {
 #'
 #' Launch Shiny example applications, and optionally, your system's web browser.
 #'
-#' @param example The name of the example to run, or \code{NA} (the default) to
+#' @param example The name of the example to run, or `NA` (the default) to
 #'   list the available examples.
 #' @param port The TCP port that the application should listen on. Defaults to
 #'   choosing a random port.
@@ -875,9 +1061,9 @@ stopApp <- function(returnValue = invisible()) {
 #'   launched automatically after the app is started. Defaults to true in
 #'   interactive sessions only.
 #' @param host The IPv4 address that the application should listen on. Defaults
-#'   to the \code{shiny.host} option, if set, or \code{"127.0.0.1"} if not.
+#'   to the `shiny.host` option, if set, or `"127.0.0.1"` if not.
 #' @param display.mode The mode in which to display the example. Defaults to
-#'   \code{showcase}, but may be set to \code{normal} to see the example without
+#'   `showcase`, but may be set to `normal` to see the example without
 #'   code or commentary.
 #'
 #' @examples
@@ -924,21 +1110,21 @@ runExample <- function(example=NA,
 
 #' Run a gadget
 #'
-#' Similar to \code{runApp}, but handles \code{input$cancel} automatically, and
+#' Similar to `runApp`, but handles `input$cancel` automatically, and
 #' if running in RStudio, defaults to viewing the app in the Viewer pane.
 #'
 #' @param app Either a Shiny app object as created by
-#'   \code{\link[=shiny]{shinyApp}} et al, or, a UI object.
-#' @param server Ignored if \code{app} is a Shiny app object; otherwise, passed
-#'   along to \code{shinyApp} (i.e. \code{shinyApp(ui = app, server = server)}).
-#' @param port See \code{\link[=shiny]{runApp}}.
+#'   [`shinyApp()`][shiny] et al, or, a UI object.
+#' @param server Ignored if `app` is a Shiny app object; otherwise, passed
+#'   along to `shinyApp` (i.e. `shinyApp(ui = app, server = server)`).
+#' @param port See [`runApp()`][shiny].
 #' @param viewer Specify where the gadget should be displayed--viewer pane,
 #'   dialog window, or external browser--by passing in a call to one of the
-#'   \code{\link{viewer}} functions.
-#' @param stopOnCancel If \code{TRUE} (the default), then an \code{observeEvent}
-#'   is automatically created that handles \code{input$cancel} by calling
-#'   \code{stopApp()} with an error. Pass \code{FALSE} if you want to handle
-#'   \code{input$cancel} yourself.
+#'   [viewer()] functions.
+#' @param stopOnCancel If `TRUE` (the default), then an `observeEvent`
+#'   is automatically created that handles `input$cancel` by calling
+#'   `stopApp()` with an error. Pass `FALSE` if you want to handle
+#'   `input$cancel` yourself.
 #' @return The value returned by the gadget.
 #'
 #' @examples
@@ -1003,10 +1189,10 @@ decorateServerFunc <- function(appobj, serverFunc) {
 #' other R environments that emulate RStudio's viewer pane/dialog APIs). If
 #' viewer APIs are not available in the current R environment, then the gadget
 #' will be displayed in the system's default web browser (see
-#' \code{\link[utils]{browseURL}}).
+#' [utils::browseURL()]).
 #'
-#' @return A function that takes a single \code{url} parameter, suitable for
-#'   passing as the \code{viewer} argument of \code{\link{runGadget}}.
+#' @return A function that takes a single `url` parameter, suitable for
+#'   passing as the `viewer` argument of [runGadget()].
 #'
 #' @rdname viewer
 #' @name viewer
@@ -1014,8 +1200,8 @@ NULL
 
 #' @param minHeight The minimum height (in pixels) desired to show the gadget in
 #'   the viewer pane. If a positive number, resize the pane if necessary to show
-#'   at least that many pixels. If \code{NULL}, use the existing viewer pane
-#'   size. If \code{"maximize"}, use the maximum available vertical space.
+#'   at least that many pixels. If `NULL`, use the existing viewer pane
+#'   size. If `"maximize"`, use the maximum available vertical space.
 #' @rdname viewer
 #' @export
 paneViewer <- function(minHeight = NULL) {
@@ -1044,7 +1230,7 @@ dialogViewer <- function(dialogName, width = 600, height = 600) {
   }
 }
 
-#' @param browser See \code{\link[utils]{browseURL}}.
+#' @param browser See [utils::browseURL()].
 #' @rdname viewer
 #' @export
 browserViewer <- function(browser = getOption("browser")) {
@@ -1062,5 +1248,5 @@ inShinyServer <- function() {
 # This check was moved out of the main function body because of an issue with
 # the RStudio debugger. (#1474)
 isEmptyMessage <- function(msg) {
-  identical(charToRaw("\003\xe9"), msg)
+  identical(as.raw(c(0x03, 0xe9)), msg)
 }
